/******************************************************************************
*   @file   ad5694r.c
*   @brief  Implementation of AD5694R Driver for NIOS processor.
*   @author ACostina (adrian.costina@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 1004
******************************************************************************/
#define AD5694R_GLOBALS

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "ad5694r.h"
#include "opencores_i2c.h"          // I2C functions.
#include <altera_avalon_pio_regs.h> // PIO access macros
#include <system.h>
#if (USE_AD5694R == 1)

/****************************************************************************/
/*  Private variables                                                       */
/****************************************************************************/
static unsigned char acknowledge            = 0;        // Acknowledge bit.
static unsigned char errorStatus            = 0;        // Error status.


/****************************************************************************/
/*  Aliases for the uC Probe data                                           */
/****************************************************************************/
volatile unsigned char* commandSlider           = &ucProbe_Data[0].uchData; // Used to set the command to be sent
volatile unsigned char* addressDacA             = &ucProbe_Data[1].uchData; // Used to set the address for DAC A
volatile unsigned char* addressDacB             = &ucProbe_Data[2].uchData; // Used to set the address for DAC B
volatile unsigned char* addressDacC             = &ucProbe_Data[3].uchData; // Used to set the address for DAC C
volatile unsigned char* addressDacD             = &ucProbe_Data[4].uchData; // Used to set the address for DAC D
volatile unsigned int*  dataSlider              = &ucProbe_Data[5].uiData;  // Used to set data to be set on DACs
volatile unsigned char* pdpuSliderA             = &ucProbe_Data[6].uchData; // Used to configure the power operation of DAC A
volatile unsigned char* pdpuSliderB             = &ucProbe_Data[7].uchData; // Used to configure the power operation of DAC B
volatile unsigned char* pdpuSliderC             = &ucProbe_Data[8].uchData; // Used to configure the power operation of DAC C
volatile unsigned char* pdpuSliderD             = &ucProbe_Data[9].uchData; // Used to configure the power operation of DAC D
volatile unsigned char* internalReferenceBtn    = &ucProbe_Data[10].uchData;// Used to configure the internal reference
volatile unsigned char* applyCommand            = &ucProbe_Data[11].uchData;// Used to send the command to the DAC
volatile unsigned char* readbackCommand         = &ucProbe_Data[12].uchData;// Used to readback data from the DAC 
volatile unsigned short* readbackDacA           = &ucProbe_Data[13].usiData;// Used to store data read from the DAC A
volatile unsigned short* readbackDacB           = &ucProbe_Data[14].usiData;// Used to store data read from the DAC B
volatile unsigned short* readbackDacC           = &ucProbe_Data[15].usiData;// Used to store data read from the DAC C
volatile unsigned short* readbackDacD           = &ucProbe_Data[16].usiData;// Used to store data read from the DAC D
volatile unsigned char* ldacButton              = &ucProbe_Data[17].uchData;// Used to control the Hardware LDAC pin
volatile unsigned char* resetButton             = &ucProbe_Data[18].uchData;// Used to control the Hardware Reset pin
volatile float* vRef                            = &ucProbe_Data[19].fData;  // Used to store the reference voltage
volatile unsigned char* vRefSelect              = &ucProbe_Data[20].uchData;// Used to select the reference voltage
volatile float* freadbackDacA                   = &ucProbe_Data[21].fData;  // Used to display output voltage for DacA
volatile float* freadbackDacB                   = &ucProbe_Data[22].fData;  // Used to display output voltage for DacB
volatile float* freadbackDacC                   = &ucProbe_Data[23].fData;  // Used to display output voltage for DacC
volatile float* freadbackDacD                   = &ucProbe_Data[24].fData;  // Used to display output voltage for DacD

/******************************************************************************
 * @brief Read the AD5694R DAC
 *
 * @param channel Select the channel to be read
 *
 * @return data - Data read from the AD5694R DAC.
******************************************************************************/
unsigned int AD5694R_Get(unsigned char channel)
{
    unsigned int data;

    I2C_start(OPENCORES_I2C_1_BASE, AD5694R_ADDRESS, 0);// Start write sequence.
    I2C_write(OPENCORES_I2C_1_BASE, channel, 1);

    I2C_start(OPENCORES_I2C_1_BASE, AD5694R_ADDRESS, 1);// Start read sequence.
    data = I2C_read(OPENCORES_I2C_1_BASE,0);            // Read first byte.
    data = data << 8;
    data = data + I2C_read(OPENCORES_I2C_1_BASE,1);     // Read second byte.
    data = data >> AD5694RDATAPOSITION; 

    return(data);
}

/******************************************************************************
 * @brief Write to the AD5694R DAC.
 *
 * @param value Data to be sent to the DAC
 *
 * @return None
******************************************************************************/
void AD5694R_Set(int value)
{
    unsigned char toSend[3];

    toSend[0] = (value >> 16 ) & 0xff;
    toSend[1] = (value >> 8 ) & 0xff;
    toSend[2] = (value) & 0xff;

    acknowledge = I2C_start(OPENCORES_I2C_1_BASE, AD5694R_ADDRESS, 0);  // Start write sequence.
    errorStatus = errorStatus | acknowledge;

    I2C_write(OPENCORES_I2C_1_BASE, toSend[0], 0);
    I2C_write(OPENCORES_I2C_1_BASE, toSend[1], 0);
    I2C_write(OPENCORES_I2C_1_BASE, toSend[2], 1);
}

/******************************************************************************
 * @brief Write to the AD5694R channel and update
 *
 * @param channel Channel or channels to which data is to be written
 * @param iDAC    Value to be set on the channel
 *
 * @return None
******************************************************************************/
void AD5694R_SetDac(int channel, int iDAC)
{

    unsigned char toSend[3];

    toSend[0] = (unsigned char)( channel) + 0x30 ;                      //command 3, channel as parameter
    toSend[1] = (iDAC >> (8 - AD5694RDATAPOSITION) ) & 0xff;
    toSend[2] = (iDAC << AD5694RDATAPOSITION ) & 0xff;

    acknowledge = I2C_start(OPENCORES_I2C_1_BASE, AD5694R_ADDRESS, 0);  // Start write sequence.
    errorStatus = errorStatus | acknowledge;

    I2C_write(OPENCORES_I2C_1_BASE, toSend[0], 0);
    I2C_write(OPENCORES_I2C_1_BASE, toSend[1], 0);
    I2C_write(OPENCORES_I2C_1_BASE, toSend[2], 1);
}

/******************************************************************************
 * @brief Initializes the communication with the device.
 *        Initializes variables with values from the default interface.
 *
 * @param None.
 *
 * @return errorStatus.
******************************************************************************/
int AD5694R_Init(void)
{
    errorStatus             = 0;
    *resetButton            = 1;
    *commandSlider          = 1;
    *internalReferenceBtn   = 1;

    IOWR_ALTERA_AVALON_PIO_DATA(GPIO_BASE, 0);                          // Reset data.

    IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASE, GPIO_LDAC | GPIO_RESET);// Output.
    IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LDAC);            // Clear LDAC pin (Active Low).
    IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_RESET);             // Set CLR pin (Active Low).

    I2C_init(OPENCORES_I2C_1_BASE, ALT_CPU_FREQ, 100000);               // Initialize I2C module.

    acknowledge = I2C_start(OPENCORES_I2C_1_BASE, AD5694R_ADDRESS, 0);  // Start write sequence.
    errorStatus = errorStatus | acknowledge;

    I2C_write(OPENCORES_I2C_1_BASE, 0x00, 0);                           // NOP
    I2C_write(OPENCORES_I2C_1_BASE, 0x00, 0);                           // NOP
    I2C_write(OPENCORES_I2C_1_BASE, 0x00, 1);                           // NOP

    return(errorStatus);
}

/******************************************************************************
 * @brief Updates the device's monitored and controlled parameters
 *
 * @param : None.
 *
 * @return: errorStatus.
******************************************************************************/
int AD5694R_UpdateParameters(void)
{
    errorStatus = 0;

    static unsigned int command;
    static unsigned int power;
    static unsigned int internalReference;
    static unsigned int ldacMask;
    static unsigned int reset;

    // Compute command words based on the ucProbeInterface

    command             = (*commandSlider << AD5694RCMDPOSITION) +\
                          ((*addressDacA + (*addressDacB << 1)  + (*addressDacC << 2) + (*addressDacD << 3)) << AD5694RCHANPOSITION) +\
                          (*dataSlider << AD5694RDATAPOSITION);
    power               = (*commandSlider << AD5694RCMDPOSITION) +\
                          (*pdpuSliderA << PWRPOSITIONA) + (*pdpuSliderB << PWRPOSITIONB) +  (*pdpuSliderC << PWRPOSITIONC) +  (*pdpuSliderD << PWRPOSITIOND);
    internalReference   = (*commandSlider << AD5694RCMDPOSITION) +\
                          (*internalReferenceBtn);
    ldacMask            = (*commandSlider << AD5694RCMDPOSITION)+\
                          *addressDacA + (*addressDacB << 1)  + (*addressDacC << 2) + (*addressDacD << 3);
    reset               = (*commandSlider << AD5694RCMDPOSITION);

    *vRef= *vRefSelect ? 5.0 : 2.5;

    // Control Hardware pins
    if(*ldacButton) // If the LDAC button is pressed, sets GPIO_LDAC pin.
    {
        IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_LDAC);
    }
    else            // Else clears GPIO_LDAC pin.
    {
        IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_LDAC);
    }

    if(*resetButton) // If the LDAC button is pressed, sets GPIO_LDAC pin.
    {
        IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASE, GPIO_RESET);
    }
    else            // Else clears GPIO_LDAC pin.
    {
        IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASE, GPIO_RESET);
    }

    // Control software registers
    if(*applyCommand)
    {
        if (*commandSlider == 4 )
        {
            AD5694R_Set(power);
        }
        else if (*commandSlider == 5)
        {
            AD5694R_Set(ldacMask);
        }
        else if (*commandSlider == 6)
        {
            AD5694R_Set(reset);
        }
        else if (*commandSlider == 7)
        {
            AD5694R_Set(internalReference);
        }
        else
        {
            AD5694R_Set(command);
        }
    }
    if(*readbackCommand)
    {
        *readbackDacA= (unsigned short) AD5694R_Get(CHANNEL_A);
        *freadbackDacA  = ((float) *vRef / (float)MAXVALUE) * (float) *readbackDacA;
        *readbackDacB= (unsigned short) AD5694R_Get(CHANNEL_B);
        *freadbackDacB  = ((float) *vRef / (float)MAXVALUE) * (float) *readbackDacB;
        *readbackDacC= (unsigned short) AD5694R_Get(CHANNEL_C);
        *freadbackDacC  = ((float) *vRef / (float)MAXVALUE) * (float) *readbackDacC;
        *readbackDacD= (unsigned short) AD5694R_Get(CHANNEL_D);
        *freadbackDacD  = ((float) *vRef / (float)MAXVALUE) * (float) *readbackDacD;
    }

    return(0);
}

#endif /* USE_AD5694R */
